#!/usr/local/bin/python

# Author                 Version      Date
# -----------------------------------------------
# S M Crawford (SAAO)    1.0          10 Jan 2008

# saltmysql are salt tasks associated with connecting to and 
# accessessing a mysql server database.


import saltprint
import os, sys, string, types, re
import MySQLdb
findNum = re.compile(r'(?:[^\d]*(\d+)[^\d]*)+')

# -----------------------------------------------------------
# Connect to  a database 

def connectdb(host,dbname,user,passwd,logfile):
    db=''
    status = 0
    try:
        db = MySQLdb.connect(host=host,db=dbname,user=user,passwd=passwd)
    except:
        message = 'ERROR: SALTMYSQL.CONNECTDB -- Cannot connect to ' + host
        status = saltprint.err(logfile,message)

    return db,status

#-----------------------------------------------------------
# Select a record from a table 

def select(db, selection, table, logic, logfile):
    status=0
    record=''
    try:
	exec_command    =""
        exec_command   +="SELECT "+selection
        exec_command   +=" FROM  "+table 
	if len(logic)>0:
         exec_command   +=" WHERE  "+logic 

	cursor = db.cursor()
	cursor.execute(exec_command)
	record = cursor.fetchall()
	cursor.close()
	
    except Exception, e:
	   message = 'ERROR: SALTMYSQL.SELECT -- Cannot read value because %s ' % e
           status = saltprint.err(logfile,message)

    return record, status

#-----------------------------------------------------------
# Insert a record into the table

def insert(db, insertion, table, logfile):
    status=0
    try:
        exec_command    =""
        exec_command   +="INSERT INTO "+table
        exec_command   +=" SET  "+insertion


        cursor = db.cursor()
        cursor.execute(exec_command)
        cursor.execute("COMMIT")

    except Exception, inst: 
           message = 'ERROR: SALTMYSQL.INSERT -- Cannot insert into ' + table + '\n'
	   message = message + inst[1]
           status = saltprint.err(logfile,message)

    return status

#-----------------------------------------------------------
# update a record in a table

def update(db, insertion, table, logic, logfile):
    status=0
    try:
        exec_command    =""
        exec_command   +="UPDATE "+table
        exec_command   +=" SET  "+insertion
	if len(logic)>0:
         exec_command   +=" WHERE  "+logic 


        cursor = db.cursor()
        cursor.execute(exec_command)
        cursor.execute("COMMIT")

    except Exception, inst:
           message = 'ERROR: SALTMYSQL.INSERT -- Cannot insert into ' + table + '\n'
           message = message + inst[1]
           status = saltprint.err(logfile,message)

    return status

#-----------------------------------------------------------
# delete a record in a table

def delete(db, table,logic, logfile):
    status=0
    try:
        exec_command    =""
        exec_command   +="DELETE FROM "+table
	if len(logic)>0:
         exec_command   +=" WHERE "+logic 


        cursor = db.cursor()
        cursor.execute(exec_command)
        cursor.execute("COMMIT")

    except Exception, inst:
           message = 'ERROR: SALTMYSQL.DELETE -- Cannot delete  entry in ' + table + '\n'
           message = message + inst
           status = saltprint.err(logfile,message)

    return status

#-----------------------------------------------------------
# create a new fits file entry in the FileData table

def createnewFileData(db, ImageHeader, FileNameString, PipelineFileNameString, logfile, verbose):
    """Create a new entry into the FileData table.

       If necessary, it will create k and put all data under there.

    """
    from salttime import datatimeobs2DateTime,sex2dec
    status=0
    message=''
    except_message=''
    FileData_Id=-1

    #set up all the image header variables
    UTStartString='1000-01-01 00:00:00'
    ExposureTimeString=''
    TelRAString=''
    TelDecString=''
    TelEpochString=''
    PipelineStartString='1000-01-01 00:00:00'
    try:
     UTStartString=ImageHeader['DATE-OBS']+' '+ImageHeader['UTC-OBS']
     ExposureTimeString='%5.3f' % ImageHeader['EXPTIME']
     TelRA=15.0*sex2dec(ImageHeader['TELRA'])
     TelDec=sex2dec(ImageHeader['TELDEC'])
     TelRAString='%10.7f' % TelRA
     TelDecString='%10.7f' % TelDec
     TelEpochString='%6.2f' % ImageHeader['EQUINOX']
     TargetNameString=ImageHeader['OBJECT']
     InstrueNameString=ImageHeader['INSTRUME']
     ObsModeString=ImageHeader['OBSMODE']
     DetModeString=ImageHeader['DETMODE']
     ProposalCodeString=ImageHeader['PROPID']
     ObservatString=ImageHeader['OBSERVAT']
     ProposalString=ImageHeader['PROPOSAL']
    except Exception, inst:
	except_message=inst[0]

    try:
     PipelineStartString=datatimeobs2DateTime(ImageHeader['SAL-TLM'])
    except Exception, inst:
     except_message=inst[0]

    try:
           name=FileNameString.split('/')[-1].split('.')[0]
           date = '%s-%s-%s' % (name[1:5], name[5:7], name[7:9])
    except Exception, e:
           message='SALTMYSQL -- ERROR:  Could not determine the date for %s because %s' \
               % (FileNameString, e)
           status=saltprint.err(logfile, message)


    
    #create the insertion command for the data
    if status == 0:
       insert_command='ObsStats_Id=1,Track_Id=1,'
       insert_command += "UTStart='"+UTStartString+"',"
       insert_command += "ExposureTime='"+ExposureTimeString+"',"
       insert_command += "TelRA='"+TelRAString+"',"
       insert_command += "TelDec='"+TelDecString+"',"
       insert_command += "TelEpoch='"+TelEpochString+"',"
       insert_command += "FileName='"+FileNameString+"',"
       insert_command += "PipelineDate='"+PipelineStartString+"',"
       insert_command += "PipelineFileName='"+PipelineFileNameString+"',"
       insert_command += "Target_Name='"+TargetNameString+"',"
       insert_command += "INSTRUME='"+InstrueNameString+"',"
       insert_command += "OBSMODE='"+ObsModeString+"',"
       insert_command += "DETMODE='"+DetModeString+"',"
       insert_command += "Proposal_Code='"+ProposalCodeString+"'"

       status=insert(db,insert_command,'FileData',logfile)

    #check to see if the FileData was created
    if status==0:
      logic="FileName='%s'" % FileNameString
      records,status=select(db,'FileData_Id','FileData',logic,logfile)
      if len(records[0])==1 :
         FileData_Id=records[0][0]
      else:
         message='SALTMYSQL -- ERROR:  New FileData entry cannot be accessed for %s ' % (inname)
         status=saltprint.err(logfile, message)


    if status==0:
	message='SALTMYSQL.CREATENEWFILEDATA -- Created entry in FileData for ' + FileNameString
	saltprint.log(logfile,message,True)
    else:
       message = 'ERROR: SALTMYSQL.CREATENEWFILEDATA -- Cannot create entry ' + \
		FileNameString + ' in FileData \n'
       message = message + except_message
       status = saltprint.err(logfile,message)
	
    return FileData_Id, status


#-----------------------------------------------------------
# update a new fits file entry in the FileData table

def updateFileData(db, ImageHeader, FileData_Id, FileNameString, PipelineFileNameString, logfile):
    """Update an entry in the FileData table.

       If necessary, it will create k and put all data under there.

    """
    from salttime import datatimeobs2DateTime,sex2dec
    status=0
    message=''
    except_message=''
 
    logic='FileData_Id=%i'%FileData_Id
  
    #set up all the image header variables
    UTStartString='1000-01-01 00:00:00'
    ExposureTimeString=''
    TelRAString=''
    TelDecString=''
    TelEpochString=''
    PipelineStartString='1000-01-01 00:00:00'
    try:
     UTStartString=ImageHeader['DATE-OBS']+' '+ImageHeader['UTC-OBS']
     ExposureTimeString='%5.3f' % ImageHeader['EXPTIME']
     TelRA=15.0*sex2dec(ImageHeader['TELRA'])
     TelDec=sex2dec(ImageHeader['TELDEC'])
     TelRAString='%10.7f' % TelRA
     TelDecString='%10.7f' % TelDec
     TelEpochString='%6.2f' % ImageHeader['EQUINOX']
     TargetNameString=ImageHeader['OBJECT']
     InstrueNameString=ImageHeader['INSTRUME']
     ObsModeString=ImageHeader['OBSMODE']
     DetModeString=ImageHeader['DETMODE']
     ProposalCodeString=ImageHeader['PROPID']
    except Exception, inst:
        except_message=inst[0]
    try:
     PipelineStartString=datatimeobs2DateTime(ImageHeader['SAL-TLM'])
    except Exception, inst:
     except_message=inst[0]

    #create the insertion command for the data
    if status == 0:
       insert_command ="UTStart='"+UTStartString+"',"
       insert_command += "ExposureTime='"+ExposureTimeString+"',"
       insert_command += "TelRA='"+TelRAString+"',"
       insert_command += "TelDec='"+TelDecString+"',"
       insert_command += "TelEpoch='"+TelEpochString+"',"
       insert_command += "FileName='"+FileNameString+"',"
       insert_command += "PipelineDate='"+PipelineStartString+"',"
       insert_command += "PipelineFileName='"+PipelineFileNameString+"',"
       insert_command += "Target_Name='"+TargetNameString+"',"
       insert_command += "INSTRUME='"+InstrueNameString+"',"
       insert_command += "OBSMODE='"+ObsModeString+"',"
       insert_command += "DETMODE='"+DetModeString+"',"
       insert_command += "Proposal_Code='"+ProposalCodeString+"'"



    status=update(db,insert_command,'FileData',logic,logfile)

    return status


#-----------------------------------------------------------
# update the fits header tables

def updateFitsHeaders(db, imageStruct, FileData_Id, logfile, verbose):
    """Update the fits header tables

       returns status
    """
    from salttime import datatimeobs2DateTime,sex2dec
    status=0
    message=''
    except_message=''
    logic='FileData_Id=%i'%FileData_Id
    try:
       ImageHeader=imageStruct[0].header
       InstrueNameString=ImageHeader['INSTRUME']
       ProposalCodeString=ImageHeader['PROPID']
       ProposalString=ImageHeader['PROPOSAL']
       ObservatString=ImageHeader['OBSERVAT']
    except Exception, e:
       message='SALTMYSQL -- ERROR:  Cannot extract image header information because %s' %e
       status=saltprint.err(logfile, message)

    #Check to see if FitsHeader exists, if not, then create
    if status==0:
       records,status=select(db,'FileData_Id','FitsHeaderImage',logic,logfile)
       if not records and status==0:
         insert_command='FileData_Id=%i,' % FileData_Id
         insert_command += "PROPID='%s',"%ProposalCodeString
         insert_command += "PROPOSAL='%s',"%ProposalString
         insert_command += "OBSERVAT='%s'" % ObservatString
         status=insert(db,insert_command,'FitsHeaderImage',logfile)

    #update FistHeaderImage
    if status==0:
      status=updatefitstable(db, 'FitsHeaderImage', ImageHeader, logic,logfile)

    #create the associate FitsHeaderPipeline table
    if status==0:
       records,status=select(db,'FileData_Id','FitsHeaderPipeline',logic,logfile)
       if not records and status==0:
         insert_command='FileData_Id=%i' % FileData_Id
         status=insert(db,insert_command,'FitsHeaderPipeline',logfile)

    #update FitsHeaderPipeline Table
    if status==0:
      status=updatefitstable(db, 'FitsHeaderPipeline', ImageHeader, logic,logfile)


    #create either the assoicate FitsHeaderRSS or SALTICAM tables
    if status==0:
       insert_command='FileData_Id=%i' % FileData_Id
       if InstrueNameString=='SALTICAM':
          records,status=select(db,'FileData_Id','FitsHeaderSalticam',logic,logfile)
          if not records and status==0:
             status=insert(db,insert_command,'FitsHeaderSalticam',logfile)
       elif InstrueNameString=='RSS':
          records,status=select(db,'FileData_Id','FitsHeaderRss',logic,logfile)
          if not records and status==0:
             status=insert(db,insert_command,'FitsHeaderRss',logfile)
       else:
          message='SALTMYSQL -- WARNING:  Did not update the Instrument tables'
          print message


    #update the SALTICAM data
    if status==0 and InstrueNameString=='SALTICAM':
       status=updatefitstable(db, 'FitsHeaderSalticam', ImageHeader, logic,logfile)
    #update the RSS data
    if status==0 and InstrueNameString=='RSS':
       status=updatefitstable(db, 'FitsHeaderRss', ImageHeader, logic,logfile)

    return status

#-----------------------------------------------------------
# Upate a fits header table

def updatefitstable(db, Table, ImageHeader, logic,logfile):
    status=0
    try:
      exec_command='show columns from %s' % Table
      cursor = db.cursor()
      cursor.execute(exec_command)
      image_columns = cursor.fetchall()
      for column in image_columns:
        ColumnName=column[0]
        insertString='Empty'
        if not ColumnName=='FileData_Id':
                HeaderName=ColumnName.replace('_','-')
                if HeaderName=='DECL': HeaderName='DEC'
                if HeaderName=='PM-DEC': HeaderName='PM_DEC'
                kstatus=updateheaderinfo(db,Table,ImageHeader,ColumnName, HeaderName,column,logic,logfile)
    except Exception, e:
      message='SALTMYSQL -- ERROR:  Failed to update %s because %s' % (Table, e)
      status=saltprint.err(logfile, message)


    return status


#-----------------------------------------------------------
# update a column in the fits header tables

def updateheaderinfo(db,Table,ImageHeader, ColumnName, HeaderName,column,logic,logfile):
    """Given a column, update it with header information

    """

    try:
      HeaderString=ImageHeader[HeaderName]
    except KeyError:
      return 1

    if type(HeaderString)==types.StringType:
      HeaderString=HeaderString.strip()
      try:
         maxChar=int(findNum.search(column[1]).groups()[0])
         if len(HeaderString) > maxChar:
            HeaderString=HeaderString[:maxChar]
         insertString="%s='%s'"% (ColumnName,HeaderString)
      except:
            insertString="%s='%s'"% (ColumnName,HeaderString)
    elif type(HeaderString)==types.FloatType:
      insertString='%s=%f'% (ColumnName,HeaderString)
    elif type(HeaderString)==types.IntType:
      insertString='%s=%i'% (ColumnName,HeaderString)
    else:
        print type(HeaderString)

    status=update(db,insertString,Table,logic,logfile)
    if status==1:
       print column,HeaderName,HeaderString,insertString,type(HeaderString)

    return status

