#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# S M Crawford (SAA0)    0.1          2 Dec 2008
#
# saltplot is a basic way to create plot widgets in pyraf and salt 
# tools
#
#

import numpy
import time
import Tkinter as Tk
from pylab import *
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg



# -----------------------------------------------------------
# Plot the data 

class saltplot:
 
 
 def __init__(self, x_arr, y_arr, xlabel, ylabel, title, logfile, verbose): 
    """Plots data
   
       returns status
    """
    #set up the variables
    status=0
    maxcolumn=7
    

    self.root= Tk.Tk()
    self.root.wm_title(title)
    self.root.bind("<Destroy>", self.destroy)

    #plot the curve
    self.lfig=figure(figsize=(8,5),dpi=72)
    self.plotcurve(x_arr, y_arr, xlabel, ylabel)

    lcrow=0
    qurow=1
    
    #add light curve plot
    self.lccanvas = FigureCanvasTkAgg(self.lfig, master=self.root)
    self.lccanvas.show()
    self.lccanvas.get_tk_widget().grid(row = lcrow, column = 0, columnspan = maxcolumn, sticky = 'news')

    #add the quit button
    self.quFrame = Tk.Frame(master=self.root)
    self.quFrame.grid(row=qurow, column=0, columnspan=maxcolumn, sticky='ew')
    self.exitbutton = Tk.Button(master=self.quFrame, text='Quit', command=self.exit)
    self.exitbutton.grid(row=0, column=3, sticky='ew')
	

    self.root.update()
    return 


 def runplotdata(self):

    Tk.mainloop()

    return 
    

 def destroy(self, e): 
    self.root.quit()
    return 

 def exit(self):
    self.root.quit()
    return

 def plotcurve(self, xarr, yarr, xlabel, ylabel): 
    """Plot the light curve 
       return ax
    """
    self.curve_plot = self.lfig.add_axes([0.10,0.10,0.8,0.80], autoscale_on=False, adjustable='datalim'  )
    self.curve_plot.hold(True)

    #plot the curve
    self.curve,=self.curve_plot.plot(xarr,yarr,linewidth=0.5,linestyle='-',marker='',color='b')

    #set up the variables
    self.lcx1=xarr.min()
    self.lcx2=xarr.max()
    self.lcy1=yarr.min()
    self.lcy2=yarr.max()
    #set up the axis
    self.curve_plot.set_xlim(self.lcx1, self.lcx2)
    self.curve_plot.set_ylim(self.lcy1, self.lcy2)
    self.curve_plot.set_ylabel(ylabel)
    self.curve_plot.set_xlabel(xlabel)



 

