#!/usr/bin/env python

# Author                 Version      Date
# -----------------------------------------------
# S M Crawford (SAAO)    0.3          06 Sep 2008

# saltstat contains statistical functions

import saltprint, numpy


# -----------------------------------------------------------
# calculate mean of numeric list

def mean(list):

    total = 0
    for item in list:
	total += item
    mean = total / len(list)
    return mean

# -----------------------------------------------------------
# calculate median of numeric list

def median(list,logfile):

    list.sort()
    n = len(list)
    if (n == 0):
	message = 'ERROR -- SALTSTAT.MEDIAN: Supplied list is empty'
	status = saltprint.err(logfile,message)
    elif (n < 3):
	median = mean(list)
    else:
	median = list[n/2]
    return median
	
# -----------------------------------------------------------
# calculate median of 2d array

def median2d(arrays,logfile):

    status = 0
    try:
	arrays = arrays.ravel()
	median = numpy.median(arrays)
    except Exception, e:
	median=None
	message = 'ERROR -- SALTSTAT.MEDIAN2D: Cannot median image arrays because %s' % e
	status = saltprint.err(logfile,message)

    return median, status

# -----------------------------------------------------------
# calculate mean of 2d array

def mean2d(arrays):

    mean = arrays[0]
    for image in arrays[1:]:
	mean += image
    mean /= len(arrays)
    
    return mean

# -----------------------------------------------------------
# calculate clipped std of 2d array

def std2dclip(arrays, mean, std, sig):
    import math
    if numpy.size(arrays)==0: return 0
    mask=((arrays-mean) < sig*std)*(arrays-mean > -sig*std)
    nsize=numpy.sum(mask)
    if nsize > 0:
     mean=numpy.sum(arrays*mask)/nsize 
     stddev=math.sqrt(numpy.sum(((arrays-mean)*mask)**2)/nsize)
    else:
     return 0
    return stddev


# -----------------------------------------------------------
# calculate the sigma clipped mean of 2d array

def mean2dclip(arrays, mean, std, sig):
    if numpy.size(arrays)==0: return 0
    mask=(arrays-mean < sig*std)*(arrays-mean > -sig*std)
    if numpy.sum(mask) > 0:
     mean=numpy.sum(arrays*mask)/numpy.sum(mask)
    else:
     return 0
    return mean

# -----------------------------------------------------------
# calculate the sigma clipped median of 2d array

def median2dclip(arr, mean, std, sig):
    if numpy.size(arr)==0: return 0
    try:
      arr = arr.ravel()
      mask=(arr-mean < sig*std)*(arr-mean > -sig*std)
      arr=numpy.compress(mask, arr)
      median = numpy.median(arr)
    except Exception, e:
     median=-1
    return median



# -----------------------------------------------------------
# Calculate image statistics

def iterstat(arr, sig, iter):
    """iterstas calculates an arrays statistics using
       a sigma clipped values
    """
    mean=arr.mean()
    std=arr.std()
    median=median2dclip(arr, mean, std, sig)
    for i in range(iter):
	mean=mean2dclip(arr, mean, std, sig)
        std=std2dclip(arr, mean, std, sig)
        median=median2dclip(arr, mean, std, sig)

    return mean, median, std

# -----------------------------------------------------------
# Median combine a set of arrays

def median_combine(arrays, logfile):

    status = 0
    try:
	median = numpy.median(arrays)
    except Exception, e:
	median=None
	message = 'ERROR -- SALTSTAT.MEDIAN_COMBINE: Cannot median combine arrays because %s' % e
	status = saltprint.err(logfile,message)

    return median, status
# -----------------------------------------------------------
# Median filter an array 
# see slotbackground more up to date version

def median_image(arr, bin, logfile):
    from scipy.ndimage.filters import median_filter
    status=0
    try:
	arr=median_filter(arr,size=(bin,bin))
    except:
	message="SALTSTAT--ERROR:  Could not median filter image"
        status=saltprint.err(logfile, message)
    return arr

