#!/usr/local/bin/python

# Author                 Version      Date
# -----------------------------------------------
# S M Crawford (SAAO)    0.3          10 Apr 2008

# salttran contains tasks involving the transformation of images


import saltprint
import numpy, pyfits

# -----------------------------------------------------------
# Embed an array in a larger array

def embed(arr, x0, y0, emarr, logfile):
    """Directly embed a 2-D array into a larger array"""
    status=0
    try:
	ylen=len(arr)
	xlen=len(arr[0])
	section='0:%i,0:%i' % (ylen, xlen)
	x1=x0
	x2=x1+xlen
	y1=y0
	y2=y1+ylen
	emarr[y1:y2,x1:x2]=arr
    except Exception, e:
        message = 'ERROR: SALTTRAN.EMBED--Could not embed array because %s ' % e
        status = saltprint.err(logfile,message)

    return emarr,status

#-----------------------------------------------------------
# shift the data in x-y 

def xyshift(arr, xshift, yshift, logfile):
    """Shift the data by an integer x and y value"""
    narr=arr
    status=0
    x0=0
    y0=0
    try:
	ylen=len(arr)
	xlen=len(arr[0])
	xlen=xlen+abs(xshift)
	ylen=ylen+abs(yshift)
	if xshift > 0: x0=xshift
	if yshift > 0: y0=yshift
	narr=numpy.zeros((ylen,xlen), numpy.float64)
	narr=embed(arr, x0, y0, narr, logfile)
    except:
	 message = 'ERROR: SALTTRAN.XYSHIFT -- Could not shift array'
         status = saltprint.err(logfile,message)

    return narr, status 

#-----------------------------------------------------------
# rebin the data

def rebin_factor(a, newshape, logfile):
    """Rebin the data to a newshape"""

    status=0
    try:
         assert len(a.shape) == len(newshape)

         slices = [ slice(0,old, float(old)/new) for old,new in zip(a.shape,newshape) ]
         coordinates = numpy.mgrid[slices]
         indices = coordinates.astype('i')   #choose the biggest smaller integer index
	 b=a[tuple(indices)]
    except Exception, e:
	 b=a
	 message='SALTTRAN--ERROR:  Unable to rebin the data because %s' %e
	 status=saltprint.err(logfile, message) 
    return b,status

#-----------------------------------------------------------
# block average the data

def blockave ( a, bin, newshape, logfile):
        """block average the image"""
	status=0
        bin=int(bin)
        if bin<=0: 
	  message='SALTTRAN.AVBIN--ERROR:  Bin size much be positive'
	  status=saltprint.err(logfile, message)
	  return a, status

	try:
          shape=(newshape[0]/bin,newshape[1]/bin)
	  dt=a.dtype
          b=numpy.zeros(shape, dtype=dt)
          for i in range(bin):
            for j in range(bin):
              d=bin*numpy.indices(shape)
              d[0]=d[0]+i
              d[1]=d[1]+j
              b += a[tuple(d)]
	  b=b/bin**2
	except Exception, e:
	  b=a
	  message='SALTTRAN.AVBIN--ERROR: Unable to block average array becase %s ' % e
	  status=saltprint.err(logfile,message)

        return b, status

